import argparse
import circlator

class Error (Exception): pass

def run():
    parser = argparse.ArgumentParser(
        description = 'Downloads and filters a file of dnaA (or other) genes from uniprot',
        usage = 'circlator get_dnaa [options] <output prefix>')
    parser.add_argument('--min_length', type=int, help='Minimum length in amino acids [%(default)s]', default=333, metavar='INT')
    parser.add_argument('--max_length', type=int, help='Maximum length in amino acids [%(default)s]', default=500, metavar='INT')
    parser.add_argument('--uniprot_search', help='Uniprot search term [%(default)s]', default='dnaa', metavar='STRING')
    parser.add_argument('--name_re', help='Each sequence name must match this regular expression [%(default)s]', default='"Chromosomal replication initiat(or|ion) protein.*dnaa"', metavar='STRING')
    parser.add_argument('--name_re_case_sensitive', action='store_true', help='Do a case-sensitive match to regular expression given by --name_re. Default is to ignore case.')

    parser.add_argument('outprefix', help='Prefix of output files', metavar='output prefix')
    options = parser.parse_args()

    dna_getter = circlator.dnaa.UniprotDownloader(
        min_gene_length=options.min_length,
        max_gene_length=options.max_length,
        uniprot_search=options.uniprot_search,
        header_regex=options.name_re,
        header_regex_ignorecase=(not options.name_re_case_sensitive)
    )

    dna_getter.run(options.outprefix)



import re
import sys
from circlator import program, common
import shutil
import pyfastaq

class Error (Exception): pass


# Travis is using python3.4, and actually "python" in travis means
# python3.4, not python2. SPAdes throws an error about not being
# compatible with python3.4.
# This means we need to explicitly run SPAdes with python2.
class Spades(program.Program):
    def exe(self):
        return 'python2 ' + shutil.which(self.path)


prog_to_env_var = {
    'samtools': 'CIRCLATOR_SAMTOOLS',
    'spades': 'CIRCLATOR_SPADES',
}


prog_to_version_cmd = {
    'bwa': ('', re.compile('^Version: ([0-9\.]+)')),
    'nucmer': ('--version', re.compile('^NUCmer \(NUCleotide MUMmer\) version ([0-9\.]+)')),
    'prodigal': ('-v', re.compile('^Prodigal V([0-9\.]+):')),
    'samtools': ('', re.compile('^Version: ([0-9\.]+)')),
    'spades': ('', re.compile('^SPAdes genome assembler v.([0-9\.]+)')),
}


min_versions = {
    'bwa': '0.7.12',
    'nucmer': '3.1',
    'prodigal': '2.6',
    'samtools': '0.1.19',
    'spades': '3.5.0',
}


max_versions = {
    'spades': '3.6.0'
}

prog_name_to_default = {
    'bwa': 'bwa',
    'nucmer': 'nucmer',
    'prodigal': 'prodigal',
    'spades': 'spades.py',
    'samtools': 'samtools',
}


prog_builders = {
    'spades': Spades
}


def handle_error(message, raise_error=True):
    if raise_error:
        raise Error(message + '\nCannot continue')
    else:
        print(message)


def make_and_check_prog(name, verbose=False, raise_error=True, filehandle=None):
    builder = prog_builders.get(name, program.Program)
    p = builder(
        prog_name_to_default[name],
        prog_to_version_cmd[name][0],
        prog_to_version_cmd[name][1],
        environment_var=prog_to_env_var.get(name, None)
    )

    if not p.in_path():
        handle_error("Didn't find " + name + " in path. Looked for:" + p.path, raise_error=raise_error)
        return p

    version = p.version()

    if version is None:
        handle_error('Found ' + name + " but couldn't get version.", raise_error=raise_error)
        return p

    if not p.version_at_least(min_versions[name]):
        handle_error('Version of ' + name + ' too low. I found ' + p.version() + ', but must be at least ' + min_versions[name] + '. Found here:\n' + p.full_path, raise_error=raise_error)
        return p

    if name != 'spades' and name in max_versions and not p.version_at_most(max_versions[name]):
        handle_error('Version of ' + name + ' too high. I found ' + p.version() + ', but must be at most ' + max_versions[name] + '. Found here:\n' + p.full_path, raise_error=raise_error)
        return p

    if name == 'spades' and not p.version_at_most(max_versions['spades']):
        print('WARNING: SPAdes version', p.version(), 'found, which is newer than 3.6.0.', file=sys.stderr)
        print('         Circlator can use this version, but some circularizations may be missed.', file=sys.stderr)
        print('         We recommend that you use version 3.6.0, which is available for Linux using:', file=sys.stderr)
        print('         wget http://spades.bioinf.spbau.ru/release3.6.0/SPAdes-3.6.0-Linux.tar.gz', file=sys.stderr)


    if verbose:
        print('Using', name, 'version', p.version(), 'as', p.full_path)

    if filehandle:
        print('Using', name, 'version', p.version(), 'as', p.full_path, file=filehandle)

    return p


def check_all_progs(verbose=False, raise_error=False, filehandle=None):
    if filehandle is not None:
        print(sys.argv[0], 'version', common.version, file=filehandle)

    for prog in sorted(prog_name_to_default):
        make_and_check_prog(prog, verbose=verbose, raise_error=raise_error, filehandle=filehandle)




import os
import shutil
import tempfile
import pymummer
import pyfastaq

class Error (Exception): pass

class Cleaner:
    def __init__(self,
        infile,
        outprefix,
        min_contig_length=2000,
        min_contig_percent_match=95,
        nucmer_diagdiff=25,
        nucmer_min_id=95,
        nucmer_min_length=500,
        nucmer_breaklen=500,
        keepfile=None,
        verbose=False
    ):
        self.infile = os.path.abspath(infile)
        self.outprefix = os.path.abspath(outprefix)
        self.min_contig_length = min_contig_length
        self.min_contig_percent_match = min_contig_percent_match
        self.contigs_to_keep = self._get_contigs_to_keep(keepfile)
        self.nucmer_diagdiff = nucmer_diagdiff
        self.nucmer_min_id = nucmer_min_id
        self.nucmer_min_length = nucmer_min_length
        self.nucmer_breaklen = nucmer_breaklen
        self.verbose = verbose


    def _get_contigs_to_keep(self, filename):
        '''Returns a set of names from file called filename. If filename is None, returns an empty set'''
        if filename is None:
            return set()

        with open(filename) as f:
            return {line.rstrip() for line in f}


    def _remove_small_contigs(self, infile, outfile, keep=None):
        '''Writes a new file with small contigs removed.
           Returns lists of all names and names of removed contigs'''
        removed = set()
        all_names = set()
        if keep is None:
            keep = set()

        file_reader = pyfastaq.sequences.file_reader(infile)
        fout = pyfastaq.utils.open_file_write(outfile)

        for seq in file_reader:
            all_names.add(seq.id)
            if len(seq) >= self.min_contig_length or seq.id in keep:
                print(seq, file=fout)
            else:
                removed.add(seq.id)

        pyfastaq.utils.close(fout)
        return all_names, removed


    def _run_nucmer(self, infile, outfile):
        '''Run nucmer of assembly against itself'''
        n = pymummer.nucmer.Runner(
            infile,
            infile,
            outfile,
            min_id=self.nucmer_min_id,
            min_length=self.nucmer_min_length,
            diagdiff=self.nucmer_diagdiff,
            maxmatch=True,
            breaklen=self.nucmer_breaklen,
            simplify=False,
            verbose=self.verbose
        )
        n.run()


    def _load_nucmer_hits(self, infile):
        '''Returns two dictionaries:
           1) name=>contig length.
           2) Second is dictionary of nucmer hits (ignoring self matches).
              contig name => list of hits'''
        hits = {}
        lengths = {}
        file_reader = pymummer.coords_file.reader(infile)
        for al in file_reader:
            if al.qry_name == al.ref_name:
                continue
            elif al.qry_name not in hits:
                hits[al.qry_name] = []
            hits[al.qry_name].append(al)
            lengths[al.qry_name] = al.qry_length
            lengths[al.ref_name] = al.ref_length
        return lengths, hits


    def _contains(self, hit):
        '''Returns True iff (the query contig is contained in the reference contig and
           the query contig is not flagged to be kept)'''
        return (
            hit.qry_name not in self.contigs_to_keep
            and hit.qry_name != hit.ref_name
            and (100 * hit.hit_length_qry / hit.qry_length >= self.min_contig_percent_match)
            and hit.percent_identity >= self.nucmer_min_id
        )


    def _containing_contigs(self, hits):
        '''Given a list of hits, all with same query,
           returns a set of the contigs containing that query'''
        return {hit.ref_name for hit in hits if self._contains(hit)}


    def _get_containing_contigs(self, hits_dict):
        '''Given dictionary of nucmer hits (made by self._load_nucmer_hits()), returns a dictionary.
           key=contig name. Value = set of contigs that contain the key.'''
        containing = {}

        for qry_name in hits_dict:
            d = self._containing_contigs(hits_dict[qry_name])
            if len(d):
                containing[qry_name] = d

        return containing


    def _get_all_containing(self, containing_contigs, name, exclude=None):
        '''containing_contigs is a dict:
             key=contig name. Value = set of contigs that contain the key.
           Returns alls contigs called "name" that contain that contig'''
        contains_name = set()
        if name in containing_contigs:
            for containing_contig in containing_contigs[name]:
                # if we have a contains b and b contains a, then this stops infinite recursion
                if containing_contig==exclude:
                    continue
                contains_name.add(containing_contig)
                new_names = self._get_all_containing(containing_contigs, containing_contig, exclude=name)
                new_names.discard(name)
                contains_name.update(new_names)
        return contains_name


    def _expand_containing_using_transitivity(self, containing_contigs):
        '''This uses a contined in b, and b contained in c to force a contained in c.
           Just in case a contained in c wasn't already found by nucmer'''
        for name in containing_contigs:
            containing_contigs[name] = self._get_all_containing(containing_contigs, name)
        return containing_contigs


    def _collapse_list_of_sets(self, sets):
        '''Input is a list of sets. Merges any intersecting sets in the list'''
        found = True
        while found:
            found = False
            to_intersect = None
            for i in range(len(sets)):
                for j in range(len(sets)):
                    if i == j:
                        continue
                    elif sets[i].intersection(sets[j]):
                        to_intersect = i, j
                        break

                if to_intersect is not None:
                    break

            if to_intersect is not None:
                found = True
                sets[i].update(sets[j])
                sets.pop(j)

        return sets


    def _get_identical_contigs(self, hits_dict):
        '''Input is a dict:
             key=contig name. Value = set of contigs that contain the key.
           Returns a list of sets of contigs that are equivalent'''
        equivalent_contigs = []

        for qry_name, containing in hits_dict.items():
            equivalent = set()
            for containing_name in containing:
                if containing_name in hits_dict and qry_name in hits_dict[containing_name]:
                    equivalent.add(containing_name)
                    equivalent.add(qry_name)

            if len(equivalent):
                equivalent_contigs.append(equivalent)
                equivalent_contigs = self._collapse_list_of_sets(equivalent_contigs)

        return equivalent_contigs


    def _longest_contig(self, contig_set, contig_lengths):
        '''Returns the name of the longest contig, from the set of names contig_set. contig_lengths
           is expected to be a dictionary of contig name => length.'''
        longest_name = None
        max_length = -1
        for name in contig_set:
            if contig_lengths[name] > max_length:
                longest_name = name
                max_length = contig_lengths[name]
        assert max_length != -1
        assert longest_name is not None
        return longest_name


    def _remove_identical_contigs(self, containing_contigs, contig_lengths):
        '''Input is dictionary of containing contigs made by self._expand_containing_using_transitivity().
           Removes redundant identical contigs, leaving one representative (the longest) of
           each set of identical contigs.
           Returns new version of dictionary, and a dictionary of contig name => contig it was replaced with'''
        identical_contigs = self._get_identical_contigs(containing_contigs)
        to_replace = {}  # contig name => name to replace it with
        for contig_set in identical_contigs:
            longest_contig = self._longest_contig(contig_set, contig_lengths)
            for name in contig_set - {longest_contig}:
                assert name not in to_replace
                to_replace[name] = longest_contig

        for name, replace_with in to_replace.items():
            if replace_with not in containing_contigs:
                containing_contigs[replace_with] = set()

            if name in containing_contigs:
                containing_contigs[replace_with].update(containing_contigs[name])
                del containing_contigs[name]

        to_delete = set()

        for name, names_set in containing_contigs.items():
            assert name not in to_replace
            new_set = {to_replace.get(x, x) for x in names_set}
            new_set.discard(name)
            if len(new_set) > 0:
                containing_contigs[name] = new_set
            else:
                to_delete.add(name)

        for name in to_delete:
            del containing_contigs[name]

        return containing_contigs, to_replace


    def _clean_contigs(self, infile, outfile, containing_contigs, replaced_contigs):
        file_reader = pyfastaq.sequences.file_reader(infile)
        fout = pyfastaq.utils.open_file_write(outfile)

        for seq in file_reader:
            if seq.id not in replaced_contigs and seq.id not in containing_contigs:
                print(seq, file=fout)

        pyfastaq.utils.close(fout)


    def _write_log(self, outfile, prefix, all_contigs, small_removed, containing_contigs, replaced_contigs):
        f = pyfastaq.utils.open_file_write(outfile)

        for name in sorted(all_contigs):
            if name in self.contigs_to_keep:
                print(prefix, name, 'user_kept', sep='\t', file=f)
            elif name in small_removed:
                print(prefix, name, 'small_removed', sep='\t', file=f)
            elif name in containing_contigs:
                print(prefix, name, 'contained in', '\t'.join(sorted(containing_contigs[name])), sep='\t', file=f)
            elif name in replaced_contigs:
                print(prefix, name, 'replaced with', replaced_contigs[name], sep='\t', file=f)
            else:
                print(prefix, name, 'kept', sep='\t', file=f)

        pyfastaq.utils.close(f)


    def run(self):
        removed_small_file = self.outprefix + '.remove_small.fa'
        names_all, names_small = self._remove_small_contigs(self.infile, removed_small_file, keep=self.contigs_to_keep)
        nucmer_coords_file = self.outprefix + '.coords'
        self._run_nucmer(removed_small_file, nucmer_coords_file)
        contig_lengths, nucmer_hits = self._load_nucmer_hits(nucmer_coords_file)
        containing_contigs = self._get_containing_contigs(nucmer_hits)
        if self.verbose and len(containing_contigs) > 0:
            print('\nContig\tContained in')
            for x in containing_contigs:
                print(x, containing_contigs[x])
            print()

        containing_contigs = self._expand_containing_using_transitivity(containing_contigs)
        containing_contigs, replaced_contigs = self._remove_identical_contigs(containing_contigs, contig_lengths)
        self._clean_contigs(removed_small_file, self.outprefix + '.fasta', containing_contigs, replaced_contigs)
        self._write_log(self.outprefix + '.log', '[clean]', names_all, names_small, containing_contigs, replaced_contigs)




__all__ = [
    'assemble',
    'bamfilter',
    'clean',
    'common',
    'dnaa',
    'external_progs',
    'fixstart',
    'mapping',
    'merge',
    'minimus2',
    'program',
    'tasks',
]

from circlator import *




import os
import shutil
import tempfile
import circlator
from bio_assembly_refinement import contig_break_finder

class Error (Exception): pass

class StartFixer:
    def __init__(self,
        assembly_fa,
        outprefix,
        min_percent_identity=70,
        genes_fa=None,
        ignore=None,
    ):
        if genes_fa is None:
            d = os.path.abspath(os.path.dirname(circlator.__file__))
            self.genes_fa = os.path.join(d, 'data', 'dnaA.fasta')
            assert os.path.exists(self.genes_fa)
        else:
            self.genes_fa = os.path.abspath(genes_fa)

        self.min_percent_identity = min_percent_identity
        self.assembly_fa = os.path.abspath(assembly_fa)
        self.outprefix = os.path.abspath(outprefix)

        if ignore is None:
            self.ignore = None
        else:
            self.ignore = os.path.abspath(ignore)


    def run(self):
        original_dir = os.getcwd()
        tmpdir = tempfile.mkdtemp(prefix=self.outprefix + '.tmp.circlator.fixstart.', dir=os.getcwd())
        os.chdir(tmpdir)

        break_finder = contig_break_finder.ContigBreakFinder(
            fasta_file=self.assembly_fa,
            gene_file=self.genes_fa,
            hit_percent_id=self.min_percent_identity,
            skip=self.ignore,
            rename=False,
            summary_prefix='[fixstart]',
        )
        break_finder.run()

        # move output files and delete tmp dir
        assert os.path.exists(break_finder.output_file)
        assert os.path.exists(break_finder.summary_file)
        os.rename(break_finder.output_file, self.outprefix + '.fasta')
        os.rename(break_finder.summary_file, self.outprefix + '.log')
        os.chdir(original_dir)
        shutil.rmtree(tmpdir)



import argparse
import os
import sys
import pyfastaq
import circlator

class Error (Exception): pass


def print_message(m, opts):
    if opts.verbose:
        print(m)


def run():
    parser = argparse.ArgumentParser(
        description = 'Run mapreads, bam2reads, assemble, merge, clean, fixstart',
        usage = 'circlator all [options] <assembly.fasta> <reads.fasta> <output directory>')
    parser.add_argument('--threads', type=int, help='Number of threads [%(default)s]', default=1, metavar='INT')
    parser.add_argument('--verbose', action='store_true', help='Be verbose')
    parser.add_argument('assembly', help='Name of original assembly', metavar='assembly.fasta')
    parser.add_argument('reads', help='Name of corrected reads FASTA file', metavar='reads.fasta')
    parser.add_argument('outdir', help='Name of output directory (must not already exist)', metavar='output directory')

    mapreads_group = parser.add_argument_group('mapreads options')
    mapreads_group.add_argument('--bwa_opts', help='BWA options, in quotes [%(default)s]', default='-x pacbio', metavar='STRING')

    bam2reads_group = parser.add_argument_group('bam2reads options')
    bam2reads_group.add_argument('--b2r_discard_unmapped', action='store_true', help='Use this to not keep unmapped reads')
    bam2reads_group.add_argument('--b2r_only_contigs', help='File of contig names (one per line). Only reads that map to these contigs are kept (and unmapped reads, unless --b2r_discard_unmapped is used). Note: the whole assembly is still used as a reference when mapping', metavar='FILENAME')
    bam2reads_group.add_argument('--b2r_length_cutoff', type=int, help='All reads mapped to contigs shorter than this will be kept [%(default)s]', default=100000, metavar='INT')
    bam2reads_group.add_argument('--b2r_min_read_length', type=int, help='Minimum length of read to output [%(default)s]', default=250, metavar='INT')

    assemble_group = parser.add_argument_group('assemble options')
    parser.add_argument('--assemble_spades_k', help='Comma separated list of kmers to use when running SPAdes. Max kmer is 127 and each kmer should be an odd integer [%(default)s]', default='127,117,107,97,87,77', metavar='k1,k2,k3,...')
    parser.add_argument('--assemble_spades_use_first', action='store_true', help='Use the first successful SPAdes assembly. Default is to try all kmers and use the assembly with the largest N50')

    merge_group = parser.add_argument_group('merge options')
    merge_group.add_argument('--merge_diagdiff', type=int, help='Nucmer diagdiff option [%(default)s]', metavar='INT', default=25)
    merge_group.add_argument('--merge_min_id', type=float, help='Nucmer minimum percent identity [%(default)s]', metavar='FLOAT', default=95)
    merge_group.add_argument('--merge_min_length', type=int, help='Minimum length of hit for nucmer to report [%(default)s]', metavar='INT', default=500)
    merge_group.add_argument('--merge_min_length_merge', type=int, help='Minimum length of nucmer hit to use when merging [%(default)s]', metavar='INT', default=4000)
    merge_group.add_argument('--merge_min_spades_circ_pc', type=float, help='Min percent of contigs needed to be covered by nucmer hits to spades circular contigs [%(default)s]', metavar='FLOAT', default=95)
    merge_group.add_argument('--merge_breaklen', type=int, help='breaklen option used by nucmer [%(default)s]', metavar='INT', default=500)
    merge_group.add_argument('--merge_ref_end', type=int, help='max distance allowed between nucmer hit and end of input assembly contig [%(default)s]', metavar='INT', default=15000)
    merge_group.add_argument('--merge_reassemble_end', type=int, help='max distance allowed between nucmer hit and end of reassembly contig [%(default)s]', metavar='INT', default=1000)
    merge_group.add_argument('--no_pair_merge', action='store_true', help='Do not merge pairs of contigs when running merge task')

    clean_group = parser.add_argument_group('clean options')
    clean_group.add_argument('--clean_min_contig_length', type=int, help='Contigs shorter than this are discarded (unless specified using --keep) [%(default)s]', default=2000, metavar='INT')
    clean_group.add_argument('--clean_min_contig_percent', type=int, help='If length of nucmer hit is at least this percentage of length of contig, then contig is removed. (unless specified using --keep) [%(default)s]', default=95, metavar='FLOAT')
    clean_group.add_argument('--clean_diagdiff', type=int, help='Nucmer diagdiff option [%(default)s]', metavar='INT', default=25)
    clean_group.add_argument('--clean_min_nucmer_id', type=float, help='Nucmer minimum percent identity [%(default)s]', metavar='FLOAT', default=95)
    clean_group.add_argument('--clean_min_nucmer_length', type=int, help='Minimum length of hit for nucmer to report [%(default)s]', metavar='INT', default=500)
    clean_group.add_argument('--clean_breaklen', type=int, help='breaklen option used by nucmer [%(default)s]', metavar='INT', default=500)

    fixstart_group = parser.add_argument_group('fixstart options')
    fixstart_group.add_argument('--genes_fa', help='FASTA file of genes to search for to use as start point', metavar='FILENAME')
    fixstart_group.add_argument('--fixstart_min_id', type=float, help='Minimum percent identity of promer match to dnaA gene [%(default)s]', default=70, metavar='FLOAT')

    options = parser.parse_args()

    print_message('{:_^79}'.format(' Checking external programs '), options)
    circlator.external_progs.check_all_progs(verbose=options.verbose, raise_error=True)

    files_to_check = [options.assembly, options.reads]
    if options.b2r_only_contigs:
        files_to_check.append(options.b2r_only_contigs)
        options.b2r_only_contigs = os.path.abspath(options.b2r_only_contigs)

    if options.genes_fa:
        files_to_check.append(options.genes_fa)

    circlator.common.check_files_exist(files_to_check)

    if options.genes_fa:
        options.genes_fa = os.path.abspath(options.genes_fa)

    original_assembly = os.path.abspath(options.assembly)
    original_reads = os.path.abspath(options.reads)


    try:
        os.mkdir(options.outdir)
    except:
        print('Error making output directory', options.outdir, file=sys.stderr)
        sys.exit(1)

    os.chdir(options.outdir)

    with open('00.info.txt', 'w') as f:
        print(sys.argv[0], 'all', ' '.join(sys.argv[1:]), file=f)
        circlator.external_progs.check_all_progs(filehandle=f, raise_error=True)

    original_assembly_renamed = '00.input_assembly.fasta'
    bam = '01.mapreads.bam'
    filtered_reads_prefix = '02.bam2reads'
    filtered_reads =  filtered_reads_prefix + '.fasta'
    assembly_dir = '03.assemble'
    reassembly = os.path.join(assembly_dir, 'contigs.fasta')
    merge_prefix = '04.merge'
    merged_fasta = merge_prefix + '.fasta'
    clean_prefix = '05.clean'
    clean_fasta = clean_prefix + '.fasta'
    fixstart_prefix = '06.fixstart'
    fixstart_fasta = fixstart_prefix + '.fasta'

    pyfastaq.tasks.to_fasta(
        original_assembly,
        original_assembly_renamed,
        strip_after_first_whitespace=True,
        check_unique=True
    )

    #-------------------------------- mapreads -------------------------------
    print_message('{:_^79}'.format(' Running mapreads '), options)
    circlator.mapping.bwa_mem(
      original_assembly_renamed,
      original_reads,
      bam,
      threads=options.threads,
      bwa_options=options.bwa_opts,
      verbose=options.verbose,
    )


    #-------------------------------- bam2reads ------------------------------
    print_message('{:_^79}'.format(' Running bam2reads '), options)
    bam_filter = circlator.bamfilter.BamFilter(
        bam,
        filtered_reads_prefix,
        length_cutoff=options.b2r_length_cutoff,
        min_read_length=options.b2r_min_read_length,
        contigs_to_use=options.b2r_only_contigs,
        discard_unmapped=options.b2r_discard_unmapped,
    )
    bam_filter.run()


    #-------------------------------- assemble -------------------------------
    print_message('{:_^79}'.format(' Running assemble '), options)
    a = circlator.assemble.Assembler(
        filtered_reads,
        assembly_dir,
        threads=options.threads,
        spades_kmers=options.assemble_spades_k,
        spades_use_first_success=options.assemble_spades_use_first,
        verbose=options.verbose
    )
    a.run()


    #------------------------------ filter original assembly -----------------
    if options.b2r_only_contigs:
        print_message('{:_^79}'.format(' --b2r_only_contigs used - filering contigs '), options)
        assembly_to_use = merge_prefix + '.00.filtered_assembly.fa'
        pyfastaq.tasks.filter(original_assembly_renamed, assembly_to_use, ids_file=options.b2r_only_contigs)
    else:
        assembly_to_use = original_assembly_renamed


    #-------------------------------- merge ----------------------------------
    print_message('{:_^79}'.format(' Running merge '), options)
    if not options.no_pair_merge:
        merge_reads = filtered_reads
    else:
        merge_reads = None

    m = circlator.merge.Merger(
        assembly_to_use,
        reassembly,
        merge_prefix,
        nucmer_diagdiff=options.merge_diagdiff,
        nucmer_min_id=options.merge_min_id,
        nucmer_min_length=options.merge_min_length,
        nucmer_min_length_for_merges=options.merge_min_length_merge,
        min_spades_circular_percent=options.merge_min_spades_circ_pc,
        spades_kmers=options.assemble_spades_k,
        spades_use_first_success=options.assemble_spades_use_first,
        nucmer_breaklen=options.merge_breaklen,
        ref_end_tolerance=options.merge_ref_end,
        qry_end_tolerance=options.merge_reassemble_end,
        threads=options.threads,
        verbose=options.verbose,
        reads=merge_reads
    )
    m.run()


    #-------------------------------- clean ----------------------------------
    merge_log = merge_prefix + '.circularise.log'
    contigs_to_keep = []
    contigs_to_not_fix_start = []
    with open(merge_log) as f:
        for line in f:
            if not line.startswith('[merge circularised]\t'):
                continue
            if line.rstrip() == '\t'.join(['[merge circularised]', '#Contig', 'repetitive_deleted', 'circl_using_nucmer', 'circl_using_spades', 'circularised']):
                continue

            x, name, u, y, z, circularised = line.rstrip().split('\t')
            if circularised == '1':
                contigs_to_keep.append(name)
            else:
                contigs_to_not_fix_start.append(name)

    clean_keep_file = clean_prefix + '.contigs_to_keep'
    with open(clean_keep_file, 'w') as f:
        if len(contigs_to_keep) > 0:
            print('\n'.join(contigs_to_keep), file=f)

    not_fix_start_file = fixstart_prefix + '.contigs_to_not_change'
    with open(not_fix_start_file, 'w') as f:
        if len(contigs_to_not_fix_start) > 0:
            print('\n'.join(contigs_to_not_fix_start), file=f)

    print_message('{:_^79}'.format(' Running clean '), options)

    cleaner = circlator.clean.Cleaner(
        merged_fasta,
        clean_prefix,
        min_contig_length=options.clean_min_contig_length,
        min_contig_percent_match=options.clean_min_contig_percent,
        nucmer_diagdiff=options.clean_diagdiff,
        nucmer_min_id=options.clean_min_nucmer_id,
        nucmer_min_length=options.clean_min_nucmer_length,
        nucmer_breaklen=options.clean_breaklen,
        keepfile=clean_keep_file,
        verbose=options.verbose
    )
    cleaner.run()


    #-------------------------------- fixstart -------------------------------
    print_message('{:_^79}'.format(' Running fixstart '), options)
    fixer = circlator.fixstart.StartFixer(
        clean_fasta,
        fixstart_prefix,
        min_percent_identity=options.fixstart_min_id,
        genes_fa=options.genes_fa,
        ignore=not_fix_start_file
    )
    fixer.run()

    with open(fixstart_prefix + '.ALL_FINISHED', 'w') as f:
        pass



import argparse
import sys
import pyfastaq
import circlator

def run():
    parser = argparse.ArgumentParser(
        description = 'Make reads from mapping to be reassembled',
        usage = 'circlator bam2reads [options] <in.bam> <outprefix>')
    parser.add_argument('--discard_unmapped', action='store_true', help='Use this to not keep unmapped reads')
    parser.add_argument('--only_contigs', help='File of contig names (one per line). Only reads that map to these contigs are kept (and unmapped reads, unless --discard_unmapped is used).', metavar='FILENAME')
    parser.add_argument('--length_cutoff', type=int, help='All reads mapped to contigs shorter than this will be kept [%(default)s]', default=100000, metavar='INT')
    parser.add_argument('--min_read_length', type=int, help='Minimum length of read to output [%(default)s]', default=250, metavar='INT')
    parser.add_argument('bam', help='Name of input bam file', metavar='in.bam')
    parser.add_argument('outprefix', help='Prefix of output filenames')
    options = parser.parse_args()

    bam_filter = circlator.bamfilter.BamFilter(
        options.bam,
        options.outprefix,
        length_cutoff=options.length_cutoff,
        min_read_length=options.min_read_length,
        contigs_to_use=options.only_contigs,
        discard_unmapped=options.discard_unmapped,
    )
    bam_filter.run()




import os
import shutil
import sys
import glob
from setuptools import setup, find_packages


setup(
    name='circlator',
    version='1.1.1',
    description='circlator: a tool to circularise genome assemblies',
    packages = find_packages(),
    package_data={'circlator': ['data/*']},
    author='Martin Hunt, Nishadi De Silva',
    author_email='path-help@sanger.ac.uk',
    url='https://github.com/sanger-pathogens/circlator',
    scripts=glob.glob('scripts/*'),
    test_suite='nose.collector',
    tests_require=['nose >= 1.3'],
    install_requires=[
        'openpyxl',
        'pyfastaq >= 3.10.0',
        'pysam >= 0.8.1',
        'pymummer>=0.6.1',
        'bio_assembly_refinement>=0.4.0',
    ],
    license='GPLv3',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Programming Language :: Python :: 3 :: Only',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    ],
)



import argparse
import sys
import pyfastaq
import circlator

def run():
    parser = argparse.ArgumentParser(
        description = 'Assemble reads using SPAdes',
        usage = 'circlator assemble [options] <in.reads.fasta> <out_dir>')
    parser.add_argument('--threads', type=int, help='Number of threads [%(default)s]', default=1, metavar='INT')
    parser.add_argument('--verbose', action='store_true', help='Be verbose')
    parser.add_argument('--spades_k', help='Comma separated list of kmers to use when running SPAdes. Max kmer is 127 and each kmer should be an odd integer [%(default)s]', default='127,117,107,97,87,77', metavar='k1,k2,k3,...')
    parser.add_argument('--spades_use_first', action='store_true', help='Use the first successful SPAdes assembly. Default is to try all kmers and use the assembly with the largest N50')
    parser.add_argument('reads', help='Name of input reads FASTA file', metavar='in.reads.fasta')
    parser.add_argument('out_dir', help='Output directory (must not already exist)')
    options = parser.parse_args()

    a = circlator.assemble.Assembler(
        options.reads,
        options.out_dir,
        threads=options.threads,
        spades_kmers=options.spades_k,
        spades_use_first_success=options.spades_use_first,
        verbose=options.verbose
    )
    a.run()



import argparse
import sys
from circlator import external_progs

def run():
    parser = argparse.ArgumentParser(
        description = 'Checks all dependencies are found and are correct versions',
        usage = 'circlator progcheck')
    options = parser.parse_args()

    external_progs.check_all_progs(verbose=True, raise_error=False)
